import edu.princeton.cs.algs4.Point2D;
import edu.princeton.cs.algs4.StdDraw;

class Rectangle implements Polygon {
    // instance variables
    private double width;
    private double height;
    private Point2D position;
    private Point2D velocity;

    // constructor
    Rectangle(double w, double h) {
        width = w;
        height = h;
        position = ShapeFuntimes.randomPos(width / 2, height / 2);
        velocity = ShapeFuntimes.randomPoint(8, 8);
    }

    // accessor methods
    public double getWidth() {
        return width;
    }

    public double getHeight() {
        return height;
    }

    public double getArea() {
        return width * height;
    }

    @Override
    public Point2D getPosition() {
        return position;
    }

    @Override
    public Point2D getVelocity() {
        return velocity;
    }
    
    @Override
    public Point2D[] getVertexes() {
        Point2D[] verts = new Point2D[4];
        // clockwise starting with upper left
        verts[0] = new Point2D(position.x() - width / 2, position.y() - height / 2);
        verts[1] = new Point2D(position.x() + width / 2, position.y() - height / 2);
        verts[2] = new Point2D(position.x() + width / 2, position.y() + height / 2);
        verts[3] = new Point2D(position.x() - width / 2, position.y() + height / 2);
        return verts;
    }

    // mutator methods
    @Override
    public void setPosition(Point2D pos) {
        position = pos;
    }

    @Override
    public void setVelocity(Point2D vel) {
        velocity = vel;
    }

    // animation methods (drawing and moving the rectangle)
    @Override
    public void draw() {
        StdDraw.setPenColor(StdDraw.RED);
        StdDraw.rectangle(position.x(), position.y(), width / 2, height / 2);
    }

    @Override
    public void move(double time) {
        position = new Point2D(position.x() + velocity.x() * time,
                               position.y() + velocity.y() * time);
        // "bounce" the rectangle if it's at the edge of the canvas
        if (ShapeFuntimes.outsideCanvas(position.x(), width / 2)) {
            velocity = new Point2D(-velocity.x(), velocity.y());
        }
        if (ShapeFuntimes.outsideCanvas(position.y(), height / 2)) {
            velocity = new Point2D(velocity.x(), -velocity.y());
        }
        draw();
    }
}